-- local log = radiant.log.create_logger('OasisComponent')
local OasisComponent = class()
local Point3 = _radiant.csg.Point3
local Cube3 = _radiant.csg.Cube3
local Region3 = _radiant.csg.Region3
local WeightedSet = require 'stonehearth.lib.algorithms.weighted_set'
local rng = _radiant.math.get_default_rng()

function OasisComponent:post_activate()
	self._world_generated_listener = radiant.events.listen_once(stonehearth.game_creation, 'stonehearth:world_generation_complete', self, self._on_world_generation_complete)
end

function OasisComponent:_on_world_generation_complete()
	local location = radiant.entities.get_world_grid_location(self._entity)
	if location and not (location.y>40) then
		location = location + Point3(1,0,1)
		self:add_grass(location,"oasis_grass_3",24)
		self:add_grass(location,"oasis_grass_2",20)
		self:add_grass(location,"oasis_grass_1",16)
		self:add_water(location)
		self:add_entities(location)
	end
	radiant.entities.destroy_entity(self._entity)
end

function OasisComponent:add_entities(location)
	local oasis_entities = {
		["stonehearth:trees:palm:small"] = 1,
		["stonehearth:trees:palm:large"] = 1,
		["stonehearth:plants:sugarbell"] = 1,
		["stonehearth:plants:cactus:tulip:blue"] = 1,
		["stonehearth:plants:cactus:tulip:red"] = 1,
		["stonehearth:plants:cactus:tulip:yellow"] = 1,
		["stonehearth:plants:wax_plant:green"] = 1,
		["stonehearth:plants:wax_plant:purple"] = 1
	}
	local oasis_table = WeightedSet(rng)
	for item, weight in pairs(oasis_entities) do
		oasis_table:add(item,weight)
	end

	for i=1,30 do
		local random_entity = radiant.entities.create_entity(oasis_table:choose_random())
		local random_location = radiant.terrain.find_placement_point(location, 9, 24)
		radiant.terrain.place_entity_at_exact_location(random_entity, random_location, {force_iconic=false})
	end
end

function OasisComponent:add_water(location)
	local cube = Cube3(
		location + Point3(-8,-2,-8),
		location + Point3( 8, 0, 8)
		)
	local water_region = radiant.terrain.intersect_cube(cube)
	for cube in water_region:each_cube() do
		radiant.terrain.subtract_cube(cube)
	end
	stonehearth.hydrology:create_water_body_with_region(water_region, 1.5)
end

function OasisComponent:add_grass(location, grass_type, radius)
	local block_type = radiant.terrain.get_block_types()[grass_type]
	local cube = Cube3(
		location + Point3(-radius,-1,-radius),
		location + Point3( radius, 0, radius)
		)
	local grass_region = radiant.terrain.intersect_cube(cube)
	for cube in grass_region:each_cube() do
		radiant.terrain.add_cube(Cube3(cube.min,cube.max,block_type))
	end
end

return OasisComponent